// Тема, в которой обсуждается этот проект: https://clck.ru/JNqKg  , http://radioservice.at.ua/forum/23-27-1
// link to the topic that discusses this development: https://clck.ru/JNqKg  , http://radioservice.at.ua/forum/23-27-1

// Релиз v2.1.3. Управление нагревателями - стандартный ДИММЕР.
/*
  Редактирование SOLOway от 14 сентября 2020 года. 
  Крайнее редактирование DmitrySh 31 июля 2020 года.
  Основа от v2.0.9 snmar4enko.
  14 профилей по 9 шагов, начальная инициализация тестовых значений всех профилей,
  если EEPROM была пуста.
  ProMINI Mega328p 5v/16MHz + USB-to-UART module (or NANO, UNO).
  LCD1602 i2c / 2pcs MAX6675.
  4pcs takte button or resist keyboard / FAN power module cooling / group FAN PCB cooling.
  Коэффициенты PID выводятся на дисплей так: Kp деленный на 10, Ki, Kd деленные на 100.

  Dmitrysh:
  библиотека PID переведена на целые числа, библиотека MAX6675 переписана функцией,
  исправлен вывод данных на дисплей, размерность переменных, пределы уставок и заданий,
  реализован вывод графиков на ПК в программу irsp © (Dmitrysh ©) на период настройки станции.
  В этой версии программы графики чертятся слева направо, внизу слева отображается текущий
  профиль, считается по последним 3-м точкам примерная скорость нагрева низом.
  Вывод графиков затем можно закомментировать или игнорировать.
  ПИН ПИЩАЛКИ = D10. Изменить можно в функции void beep, не забудьте только во всём скетче поменять!
  Перед загрузкой скетча есть возможность выбрать работу с резистивной клавиатурой, или
  с обычными тактовыми кнопками, в скетче описано как.
  Параметры хранятся в виде структуры - экономит память и бережёт EEPROM.
  Теперь используется CyberLib не патченная, можно и патченную - разницы нет.
  Пины max6675 используют CyberLib, что повышает быстродействие, но не позволяет быстро
  их переназначить.
  PID_SAMPLING_TIME увеличено до 240мс(реально ПИД расчитывается через 250мс), что
  согласовывает это время со временем замера температуры.
  Избавились от фильтра Калмана, заменив на "скользящее среднее".
  Исправлено много мелких недоработок.
  Введена своеобразная защита от отвала термопары.
  Управление нагревателями - стандартный ДИММЕР. Используется алгоритм Брезенхема для
  равномерного распределения импульсов по периоду
  По завершении пайки автоматически включается охладитель платы (внешний вентилятор).
  Автоотключение внешнего вентилятора через 5 мин после завершения пайки или сразу клавишей Cancel
  02-08-2020 SOLOway переназначил пины подключения, чтобы стало совместимо с 1.3.4с.
*/

#pragma GCC optimize ("Os")                             //хитрая команда для компилятора
#include <CyberLib.h>                                   //библиотека-ускоритель скетчей
#include <EEPROM.h>                                     //библиотека энергонезависимой памяти
#include <LiquidCrystal_I2C.h>                          //библиотека дисплея LCD1602 i2c
LiquidCrystal_I2C lcd(0x3f, 16, 2);                     //вместо (0x3f) или (0x27) указываем адрес своего дисплея
#define resist_keyboard 1                               //Раскоментировать, если клавиатура резистивная

//---секция ПИД---//
double  Input1, Input2;                                 //текущая температура(измеренная max6675) ВИ и НИ соответственно
byte Output1, Output2;                                  //выход ПИД регулятора ВИ и НИ соответственно
float e1, p1, d1, e2, p2, d2;                           //ошибка регулирования, П-составляющая, Д-составляющая для ВИ и НИ соответственно
float integra, integra2;                                //интегральные составляющие ВИ и НИ
#define i_min 0.0                                       //минимум И составляющей
#define i_max 100.0                                     //максимум И составляющей
//---секция ПИД---//

//---секция алгоритма Брезенхема---//
int er1 = 1;                                            //переменная для расчета переполнения в алгоритме Брезенхема для ВИ
int er2 = 1;                                            //переменная для расчета переполнения в алгоритме Брезенхема для НИ
int reg1;                                               //промежуточная переменная для алгоритма Брезенхема для ВИ
int reg2;                                               //промежуточная переменная для алгоритма Брезенхема для НИ
boolean out1;                                           //выход импульса управления ВИ алгоритма Брезенхема
boolean out2;                                           //выход импульса управления НИ алгоритма Брезенхема
#define PID_SAMPLING_TIME 250                           //скорость пересчета PID
//---секция алгоритма Брезенхема---//

//---секция кнопок---//
//состояние кнопок по умолчанию
boolean upSwitchState = false;
boolean downSwitchState = false;
boolean cancelSwitchState = false;
boolean okSwitchState = false;
//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;
boolean  button_state1 = false;
boolean  button_long_state1 = false;
//аналоговая клавиатура на А0
#ifdef resist_keyboard
#define PIN_RIGHT 100
#define PIN_UP 101
#define PIN_DOWN 102
#define PIN_LEFT 103
#define PIN_SELECT 104

struct A_PIN_DESC {                                //определяем  структуру которой будем описывать какое значение мы ожидаем для каждого псевдо-пина
  byte pinNo;                                     //номер пина
  int expectedValue;                              //ожидаемое значение
};
A_PIN_DESC expected_values[] = {                   //ожидаемые значения для псевдо-кнопок
  { PIN_RIGHT, 0},
  { PIN_UP, 748},
  { PIN_DOWN, 503},
  { PIN_LEFT, 326},
  { PIN_SELECT, 136}
};
#define A_PINS_COUNT sizeof(expected_values)/sizeof(A_PIN_DESC) //вычисляем сколько у нас всего "псевдо-кнопок" заданно.
#define A_POSSIBLE_ABERRATION 20                                //допустимое отклонение analogRead от ожидаемого значения, при котором псевдо кнопка считается нажатой
bool digitalReadA(byte pinNo) {
  for (byte i = 0; i < A_PINS_COUNT; i++) {        //ищем описание нашего всевдо-пина
    A_PIN_DESC pinDesc = expected_values[i];       //берем очередное описание
    if (pinDesc.pinNo == pinNo) {                //нашли описание пина?
      int value = A0_Read;                     //производим чтение аналогово входа
      return (abs(value - pinDesc.expectedValue) < A_POSSIBLE_ABERRATION); //возвращаем HIGH если отклонение от ожидаемого не больше чем на A_POSSIBLE_ABERRATION
    }
  }
  return LOW;                                       //если не нашли описания - считаем что пин у нас LOW
}
//Назначаем пины кнопок управления
byte upSwitchPin = PIN_UP;
byte downSwitchPin = PIN_DOWN;
byte cancelSwitchPin = PIN_LEFT;
byte okSwitchPin = PIN_RIGHT;
#endif
//---секция кнопок---//

//---секция профиля---//
struct pr {                                                     //основные поля профиля
  byte profileSteps;                                            //количество шагов профиля
  byte Setpoint2;                                               //уставка НИ
  byte rampRateStep[9];                                         //скорость роста температуры
  byte dwellTimerStep[9];                                       //установленное время перехода на следующий шаг
  byte temperatureStep[9];                                      //заданные температуры для шагов профиля ВИ
  byte kp1;                                                     //пропорциональный коэффициент ВИ
  byte ki1;                                                     //интегральный коэффициент ВИ
  byte kd1;                                                     //дифференциальный коэффициент ВИ
  byte kp2;                                                     //пропорциональный коэффициент НИ
  byte ki2;                                                     //интегральный коэффициент НИ
  byte kd2;                                                     //дифференциальный коэффициент НИ
  byte pwr_TOP;                                                 //максимальная мощность верхнего нагревателя
  byte pwr_BOTTOM;                                              //максимальная мощность нижнего нагревателя
};
byte SizeProfile = sizeof(pr);                                  //длина поля данных
pr profile;
uint8_t currentProfile = 1;                                     //текущий профиль
uint8_t currentStep = 1;                                        //текущий шаг профиля
uint8_t editStep = 0;                                           //номер шага профиля который редактируется в данный момент
byte Setpoint1;                                                 //установленная для шага температура(уставка)ВИ
byte Setpoint22;                                                //дублёр Setpoint2 для задания условия старта ВИ
//---секция профиля---//

//---Секция флагов---//
boolean TopStart = false;                                       //флаг включения ВИ
boolean flag = false;                                           //флаг для фиксации стартовой температуры
boolean alarmOn = false;                                        //признак предупреждения
boolean updateScreen = true;                                    //признак обновления экрана
boolean chet = false;                                           //управление четностью полуволн синусоиды сети
//---Секция флагов---//

//---Спецзнак градус цельсия---//
byte GC[8] = {
  0b01100,
  0b10010,
  0b10010,
  0b01100,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

byte Th[8] = {
  0b00100,
  0b01110,
  0b11111,
  0b10101,
  0b00000,
  0b10101,
  0b00000,
  0b00000
};

byte Bh[8] = {
  B00000,
  B01001,
  B10010,
  B01001,
  B00000,
  B11111,
  B11111,
  B01110,
};
//---Спецзнак градус цельсия---//

//---Секция переменных общего назначения---//
byte startTemp;                                             //стартовая температура для профиля, когда НИ вышел на уставку, используется для определения шага профиля с которого начинаем
byte setpointRamp;                                          //заданная температура по определённому шагу профиля ВИ, связана с заданной скоростью роста температуры
byte curCount = 0;                                          //знакоместо звездочек (они бегают по LCD в режиме RUN)
volatile byte counter;                                      //счётчик для рампы и счетчик для времени нахождения в шаге
unsigned long previousMillis;                               //это для счетчиков
boolean run_external_fan = false;                           //признак состояния охладителя платы
#define SAMPLE_TIME 240                                     //частота обновления текущей температуры и ПИД (500 = 2 раза в секунду)
#define GRAPHICS_SAMPLING_TIME 2000                         //скорость движения "звёздочек" при пайке
unsigned long nextRead1;                                    //переменная для обновления текущей температуры
unsigned long nextRead2;                                    //переменная для движения "звёздочек" при пайке
unsigned long nextRead3;                                    //переменная для обновления PID
byte tc1;                                                   //переменная для хранения текущей температуры ВИ
byte tc2;                                                   //переменная для хранения текущей температуры НИ
int i = 0;
byte Secs = 0;                                              //переменная для отсчёта времени для вывода графиков при пайке
//---Секция переменных общего назначения---//

//---структура статусов событий---//
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE : byte
{
  REFLOW_STATE_IDLE,                                   //экран в режиме простоя, выбор номера профиля
  REFLOW_STATE_MENU_STEPS,                             //меню выбора числа шагов редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_HEAT,                       //меню выбора уставки НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_PWR,                        //меню выбора мощности НИ редактируемого профиля
  REFLOW_STATE_MENU_STEP_RAMP,                         //меню выбора скорости роста t° ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_STEP_TARGET,                       //меню выбора t° уставки ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR,                           //меню выбора мощности ВИ редактируемого профиля
  REFLOW_STATE_MENU_STEP_DWELL,                        //меню выбора времени удержания t° уставки ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_P,                          //меню выбора коэффициента P для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_I,                          //меню выбора коэффициента I для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_D,                          //меню выбора коэффициента D для НИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_P,                             //меню выбора коэффициента P для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_I,                             //меню выбора коэффициента I для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_D,                             //меню выбора коэффициента D для ВИ редактируемого профиля
  REFLOW_STATE_STEP_RAMP,                              //"процесс пайки", рост температуры с заданной для текущего шага ВИ скоростью
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE,
  REFLOW_STATE_MENU_RESET
}
reflowState_t;

typedef enum REFLOW_STATUS                             //структура статуса пайки, запущен или нет
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;
reflowStatus_t reflowStatus;                           //переменная статусов событий
reflowState_t reflowState;                             //переменная статуса пайки
//---структура статусов событий---//

void loadProfile()                                     //чтение профиля из EEPROM
{
  for (byte j = 0; j <= SizeProfile - 1; j++)
  {
    EEPROM.get((currentProfile - 1)*SizeProfile, profile);
  }
  Setpoint22 = profile.Setpoint2;
  return;
}

void SaveProfile () {                                  //сохранение текущего профиля
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.put((currentProfile - 1)*SizeProfile, profile);
  }
}

void initEeprom()                                      //Запись в память начальных значений
{
  if (EEPROM.read(500) == 'C')
  {
    //что-то только для уже инициализированой памяти
  } else {
    //профиль по умолчанию:
    profile =
    { 3,                                                        //шаги
      150,                                                      //температура низа
      5, 7, 5, 5, 5, 5, 5, 5, 5,                                //9 скоростей роста температуры шага
      5, 5, 15, 1, 1, 1, 1, 1, 1,                               //9 продолжительностей шага
      150, 180, 190, 190, 205, 210, 215, 220, 225,              //температуры верхнего нагревателя по 9 шагам
      70, 50, 30,                                               //PID верха, Kp умноженный на 10, Ki Kd умноженое на 10
      70, 50, 30,                                               //PID низа, Kp умноженный на 10, Ki Kd умноженое на 10
      100, 100                                                  //ограничение мощности верха и низа в процентах
    };

    EEPROM.put(0, profile);
    //Записываем метку CFG о том что в памяти уже есть значения
    EEPROM.write(500, 'C');
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("EEPROM");
    lcd.setCursor(0, 1);
    for (uint8_t i = 0; i < 11; i++)
    {
      lcd.print(".");
      delay(150);
    }
  }
}

void sReset() {
  asm volatile ("jmp 0");
}

void setup()
{
  //Описание пинов подключения

  D9_Out; D9_High;                              //cs MAX6675 датчика нижнего нагревателя
  D8_Out; D8_High;                              //cs MAX6675 датчика верхнего нагревателя
  D6_In; D6_High;                               //DO (SO) MAX6675
  D7_Out;                                       //CLK (SCK) MAX6675

  //D12 = выход управления ВЕРХНИМ нагревателем
  //D11 = выход управления НИЖНИМ нагревателем
  D12_Out; D11_Out;                             //выходы управления верхним и нижним нагревателями
  D12_Low; D11_Low;                             //верхний и нижний выходы установить на выходах низкий уровень сигнала
  D2_In;                                        //настраиваем порт на вход для отслеживания прохождения сигнала через ноль
  attachInterrupt(0, Dimming, RISING);          //настроить срабатывание прерывания interrupt0 на pin 2
  //attachInterrupt(0, detect, FALLING);        //настроить срабатывание прерывания interrupt0 на pin 2 на низкий уровень
  /*
    CHANGE – прерывание вызывается при любом изменении значения на входе;
    RISING – вызов прерывания при изменении уровня напряжения с низкого (Low) на высокий(HIGH)
    FALLING – вызов прерывания при изменении уровня напряжения с высокого (HIGH) на низкий (Low)
  */
  Serial.begin(9600);
  D10_Out;                                      //пин пищалки buzzerPin = 10
  D4_Out; D4_Low;                               //внутренний вентилятор intefan выключаем
  D5_Out; D5_Low;                               //внешний вентилятор extefan выключаем

#ifndef resist_keyboard
  //Назначаем пины кнопок управления
  D14_In; D14_High;                             //upSwitchPin, подтяжка PullUP
  D15_In; D15_High;                             //downSwitchPin, подтяжка PullUP
  D17_In; D17_High;                             //okSwitchPin, подтяжка PullUP
  D16_In; D16_High;                             //cancelSwitchPin, подтяжка PullUP
#endif

  //сообщение приветствия
  lcd.begin();
  // Для дисплея спецсимвол Градус Цельсия ....
  lcd.createChar(0, GC);
  lcd.createChar(1, Th);
  lcd.createChar(2, Bh);
  // и его печать
  // lcd.write(byte(0));
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("IR station 2.1.3");
  lcd.setCursor(0, 1);
  lcd.print(" MAX6675 DIMMER ");
  //Мелодия приветствия
  beep_(200, 523);
  beep_(200, 659);
  beep_(200, 784);
  beep_(200, 1046);

  initEeprom();
  lcd.clear();
  loadProfile();
  nextRead1 = millis();
}

char buf[32];                                         //БУФЕР для вывода графиков

void loop()
{ Start
#ifdef resist_keyboard
  //Считываем состояние кнопок управления для резистивной клавиатуры
  upSwitchState = digitalReadA(upSwitchPin);
  downSwitchState = digitalReadA(downSwitchPin);
  cancelSwitchState = digitalReadA(cancelSwitchPin);
  okSwitchState = digitalReadA(okSwitchPin);
#endif
  //Считываем состояние кнопок управления для обычных кнопок
#ifndef resist_keyboard
  upSwitchState = !D14_Read;
  downSwitchState = !D15_Read;
  cancelSwitchState = !D17_Read;
  okSwitchState = !D16_Read;
#endif

  unsigned long currentMillis = millis();

  float out_float;
  byte SP2 = Setpoint22;
  //буззер при нажатии на кнопки (нервирует), кому надо раскоментируйте ниже строки
  //  if (upSwitchState == HIGH || downSwitchState == HIGH || cancelSwitchState == HIGH || okSwitchState == HIGH)
  //  {
  //    beep_(100,1045);
  //  }

  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      for (uint8_t j = 1; j < 5; j++) {
        beep_(100, 1046);
        delay(100);
      }
      i++;
    } else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
      TopStart = false;
      currentStep = 1;
      counter = 0;
      setpointRamp = 0;
      flag = false;                                           //после остановки профиля сбрасываем флаг

      //обновляем дисплей
      if (updateScreen) {
        //Настройка экрана в режиме ожидания
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("IDLE ");
        lcd.setCursor(0, 1);
        lcd.print("Pr  ");
        lcd.setCursor(5, 0);
        lcd.write(byte(1));
        lcd.setCursor(5, 1);
        lcd.write(byte(2));
        lcd.write(byte(2));

        updateScreen = false;
      }
      lcd.setCursor(2, 1);
      lcd.print(currentProfile, HEX);

      if (profile.temperatureStep[profile.profileSteps - 2] < 181) {
        lcd.print("+");
        lcd.write(byte(2));
      } else {
        lcd.print("-");
        lcd.write(byte(2));
      }

      lcd.setCursor(7, 0);
      lcd.print(profile.temperatureStep[profile.profileSteps - 1]);
      lcd.write(byte(0));
      lcd.setCursor(7, 1);
      lcd.print(SP2);
      lcd.write(byte(0));
      if (millis() > nextRead1)
      {
        lcdTemp();
        nextRead1 = millis() + 500;
      }

      if (run_external_fan && (millis() - previousMillis) > 300000)
      {
        run_external_fan = false;
        D5_Low;                                                   //внешний вентилятор выключить
      }

      if (upSwitchState == HIGH && ( millis() - ms_button) > 500)  //если нажата кнопка вверх, нужно перейти к следующему профилю
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 15)                                  //если номер профиля = 14 и нажата кнопка вверх, нужно вернуться к профилю 1
        {
          currentProfile = 1;
        }
        loadProfile();                                             //читаем из eeprom и перенастраиваем ПИД если изменили профиль
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 500)//то же, что и выше, просто спуститесь на один профиль
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 14;
        }
        loadProfile();                                            //читаем из eeprom и перенастраиваем ПИД если изменили профиль
      }

      //фиксируем момент нажатия кнопки "ОК" + защита от дребезга
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }

      //держим "ОК" 5 секунд и заходим в меню настроек
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button) > 5000)
      {
        button_long_state = true;
        button_state = false;
        reflowState = REFLOW_STATE_MENU_STEPS;
        updateScreen = true;
      }

      //фиксируем момент нажатия кнопки "Cancel" + защита от дребезга
      if (cancelSwitchState == HIGH && !button_state1 && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state1 = true;
        button_long_state1 = false;
        D4_Low;                                              //внутренний вентилятор выключить
        D5_Low;                                              //внешний вентилятор выключить
      }

      //держим "Cancel" 8 секунд и стираем память
      if (cancelSwitchState == HIGH && !button_long_state1 && ( millis() - ms_button) > 8000)
      {
        button_long_state1 = true;
        ms_button =  millis();
        button_state1 = false;
        reflowState = REFLOW_STATE_MENU_RESET;
        updateScreen = true;
      }
      if (cancelSwitchState == LOW )
      {
        if (button_long_state1) button_long_state1 = false;
        if (button_state1) button_state1 = false;
      }

      //включаем пайку, кнопка сработает после отпускания "ОК" если нажатие меньше 3х секунд
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        beep_(500, 1045);                                         //звуковой сигнал при старте профиля
        updateScreen = true;
        sprintf (buf, "SYNC\r\n");
        Serial.print(buf);
        curCount = 0;
        integra = 0;
        integra2 = 0;
        p1 = 0;
        p2 = 0;
        d1 = 0;
        d2 = 0;
        nextRead1 = millis();
        nextRead2 = millis();
        nextRead3 = millis();
        startTemp = 0;
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
      }
      break;

    //устанавливаем количество шагов профиля
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen) {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Profile ");
        lcd.print(currentProfile);
        lcd.print(" Edit");
        lcd.setCursor(0, 1);
        lcd.print("Profile Steps: ");
        lcd.print(profile.profileSteps);
        updateScreen = false;
      }
      lcd.setCursor(15, 1);
      lcd.print(profile.profileSteps);

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.profileSteps = profile.profileSteps + 1;
        if (profile.profileSteps >= 10) {
          profile.profileSteps = 9;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.profileSteps = profile.profileSteps - 1;
        if (profile.profileSteps <= 1)
        {
          profile.profileSteps = 1;
        }
      }
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню кнопка "ОК" не считалась нажатой

      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //Стираем всю память и переходим в начало
    case REFLOW_STATE_MENU_RESET:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("EEPROM formatted");
      for (int i = 0; i < 512; i++) {
        EEPROM.write(i, 255);
      }
      sReset();
      break;

    //устанавливаем температуру "Нижнего Нагревателя"
    case REFLOW_STATE_MENU_BOTTOM_HEAT:

      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("Bottom Heat:      ");
        updateScreen = false;
      }
      lcd.setCursor(12, 1);
      lcd.print(profile.Setpoint2);
      lcd.write(byte(0));
      lcd.print(" ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.Setpoint2 = profile.Setpoint2 + 1;
        if (profile.Setpoint2 >= 255)
        {
          profile.Setpoint2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.Setpoint2 = profile.Setpoint2 - 1;
        if (profile.Setpoint2 <= 80)
        {
          profile.Setpoint2 = 80;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_BOTTOM_PWR:

      if (updateScreen)
      {
        lcd.setCursor(0, 1);
        lcd.print("Bottom PWR:     ");
        updateScreen = false;
      }
      lcd.setCursor(12, 1);
      lcd.print(profile.pwr_BOTTOM);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        profile.pwr_BOTTOM = profile.pwr_BOTTOM + 1;
        if (profile.pwr_BOTTOM >= 100)
        {
          profile.pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        if (profile.pwr_BOTTOM >= 1) profile.pwr_BOTTOM = profile.pwr_BOTTOM - 1;
        else  profile.pwr_BOTTOM = 0;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_TOP_PWR:

      if (updateScreen)
      {
        lcd.setCursor(0, 1);
        lcd.print("TOP PWR:     ");
        updateScreen = false;
      }
      lcd.setCursor(12, 1);
      lcd.print(profile.pwr_TOP);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        profile.pwr_TOP = profile.pwr_TOP + 1;
        if (profile.pwr_TOP >= 100)
        {
          profile.pwr_TOP = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        if (profile.pwr_TOP >= 1) profile.pwr_TOP = profile.pwr_TOP - 1;
        else (profile.pwr_TOP = 0);
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_RAMP;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем скорость нагрева "Верхним Нагревателем"
    case REFLOW_STATE_MENU_STEP_RAMP:

      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("Step");
        lcd.print(editStep + 1);
        lcd.print(" Ramp:    ");
        updateScreen = false;
      }
      lcd.setCursor(11, 1);
      out_float = profile.rampRateStep[editStep] / 10.0;
      lcd.print(out_float, 1);
      lcd.write(byte(0));
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.rampRateStep[editStep] = profile.rampRateStep[editStep] + 1;
        if (profile.rampRateStep[editStep] >= 30)                   //(только целые числа) максимальная скорость роста температуры умноженная на 10
        {
          profile.rampRateStep[editStep] = 30;                      //(только целые числа) максимальная скорость роста температуры умноженная на 10
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.rampRateStep[editStep] = profile.rampRateStep[editStep] - 1;
        if (profile.rampRateStep[editStep] <= 1)                    //(только целые числа) минимальная скорость роста температуры умноженная на 10
        {
          profile.rampRateStep[editStep] = 1;                       //(только целые числа) минимальная скорость роста температуры умноженная на 10
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
          SaveProfile();
        } else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем температуру "Верхнего Нагревателя"
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("Step");
        lcd.print(editStep + 1);
        lcd.print(" Target:");
        lcd.print(profile.temperatureStep[editStep]);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.print(" ");
      lcd.setCursor(13, 1);
      lcd.print(profile.temperatureStep[editStep]);
      lcd.print("   ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.temperatureStep[editStep] = profile.temperatureStep[editStep] + 1;
        if (profile.temperatureStep[editStep] >= 255)
        {
          profile.temperatureStep[editStep] = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.temperatureStep[editStep] = profile.temperatureStep[editStep] - 1;
        if (profile.temperatureStep[editStep] <= 0)
        {
          profile.temperatureStep[editStep] = 0;
        }
      }

      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
          SaveProfile();
        } else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //устанавливаем время перехода на следующий шаг (длительность шага вплоть до 4-ёх минут)
    case REFLOW_STATE_MENU_STEP_DWELL:

      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print(" Dwell:");
        lcd.print(profile.dwellTimerStep[editStep]);
        lcd.print("c ");
      }
      lcd.print("  ");
      lcd.setCursor(13, 1);
      lcd.print(profile.dwellTimerStep[editStep]);
      lcd.print("c ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] + 1;
        if (profile.dwellTimerStep[editStep] >= 240)
        {
          profile.dwellTimerStep[editStep] = 240;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.dwellTimerStep[editStep] = profile.dwellTimerStep[editStep] - 1;
        if (profile.dwellTimerStep[editStep] <= 0)
        {
          profile.dwellTimerStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profile.profileSteps) {

          editStep = 0;
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
          SaveProfile();
        } else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //настройка "ПИД" нижнего нагревателя
    case REFLOW_STATE_MENU_BOTTOM_P:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("B.Heat: ");
        lcd.setCursor(8, 1);
        lcd.print("P=");
        out_float = profile.kp2 / 10.0;
        lcd.print(out_float);
        lcd.print("    ");
        updateScreen = false;
      }
      lcd.setCursor(10, 1);
      out_float = profile.kp2 / 10.0;
      lcd.print(out_float);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kp2 = profile.kp2 + 1;
        if (profile.kp2 >= 255)
        {
          profile.kp2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kp2 = profile.kp2 - 1;
        if (profile.kp2 <= 0)
        {
          profile.kp2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    case REFLOW_STATE_MENU_BOTTOM_I:
      lcd.setCursor(8, 1);
      lcd.print("I=");
      out_float = profile.ki2 / 100.0;
      lcd.print(out_float);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki2 = profile.ki2 + 1;
        if (profile.ki2 >= 255)
        {
          profile.ki2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki2 = profile.ki2 - 1;
        if (profile.ki2 <= 0)
        {
          profile.ki2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    case REFLOW_STATE_MENU_BOTTOM_D:
      lcd.setCursor(8, 1);
      lcd.print("D=");
      out_float = profile.kd2 / 100.0;
      lcd.print(out_float);
      lcd.print("      ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd2 = profile.kd2 + 1;
        if (profile.kd2 >= 255)
        {
          profile.kd2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd2 = profile.kd2 - 1;
        if (profile.kd2 <= 0)
        {
          profile.kd2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_P;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //настройка "ПИД" верхнего нагревателя
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        lcd.setCursor(0, 1);
        lcd.print("T.Heat: ");
        lcd.setCursor(8, 1);
        lcd.print("P=");
        out_float = profile.kp1 / 10.0;
        lcd.print(out_float);
        lcd.print("    ");
        updateScreen = false;
      }
      lcd.setCursor(10, 1);
      out_float = profile.kp1 / 10.0;
      lcd.print(out_float);
      lcd.print("    ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kp1 = profile.kp1 + 1;
        if (profile.kp1 >= 255)
        {
          profile.kp1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kp1 = profile.kp1 - 1;
        if (profile.kp1 <= 0)
        {
          profile.kp1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
      lcd.setCursor(8, 1);
      lcd.print("I=");
      out_float = profile.ki1 / 100.0;
      lcd.print(out_float);
      lcd.print("     ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki1 = profile.ki1 + 1;
        if (profile.ki1 >= 255)
        {
          profile.ki1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.ki1 = profile.ki1 - 1;
        if (profile.ki1 <= 0)
        {
          profile.ki1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_D;
        SaveProfile();
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    case REFLOW_STATE_MENU_TOP_D:
      lcd.setCursor(8, 1);
      lcd.print("D=");
      out_float = profile.kd1 / 100.0;
      lcd.print(out_float);
      lcd.print("     ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd1 = profile.kd1 + 1;
        if (profile.kd1 >= 255)
        {
          profile.kd1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profile.kd1 = profile.kd1 - 1;
        if (profile.kd1 <= 0)
        {
          profile.kd1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        //сохраняем текущие параметры в "Память"
        SaveProfile ();
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    //"процесс пайки", рост температуры с заданной скоростью
    case REFLOW_STATE_STEP_RAMP:
      if (updateScreen) {
        lcd.setCursor(0, 0);
        lcd.print(" RUN");
        lcd.setCursor(11, 0);
        lcd.print(" ");
        lcd.setCursor(11, 1);
        lcd.print("*");
        updateScreen = false;
      }

      if (TopStart == true)
      {
        // если включен верхний нагреватель рисуем обе звездочки
        lcd.setCursor(11, 1);
        lcd.print("*");
        lcd.setCursor(11, 0);
        lcd.print("*");
      }

      //фиксируем стартовую температуру
      if (!flag)
      {
        startTemp = tc1;
        D4_High;                                               //intefan включить
        flag = true;
      }

      lcd.setCursor(4, 0);
      lcd.print(currentStep);
      lcd.setCursor(7, 1);
      lcd.print(SP2);
      lcd.write(byte(0));
      lcd.print("*");


      //счётчик скорости роста температуры
      if ((currentMillis - previousMillis) > 1000 / (profile.rampRateStep[currentStep - 1] * 0.1)) //скорость роста температуры от 0.1с. до 3с.
      {
        previousMillis = currentMillis;
        if (setpointRamp < profile.temperatureStep[currentStep - 1])counter = counter + 1;
        setpointRamp = counter + startTemp;
        lcd.setCursor(7, 0);
        lcd.print(setpointRamp);
        lcd.write(byte(0));
        lcd.print(" ");
        if (Setpoint22 >= setpointRamp) profile.Setpoint2 = setpointRamp;
        else Setpoint1 = setpointRamp;
      }
      //----------------------------------------------------------------------------------------
      if (Input2 >= Setpoint22 - 1)                 //Когда достигута уставка НИ, или она меньше на 1°С
      {
        TopStart = true;                            //включаем верхний нагреватель
        //рисуем звездочку включения верхнего нагревателя, хотя включается по достижению
        lcd.setCursor(11, 0);
        lcd.print("*");
      }
      if (setpointRamp >= profile.temperatureStep[currentStep - 1] && (TopStart == true)) //если достигли нужной температуры
      {
        reflowState = REFLOW_STATE_STEP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        //reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_COMPLETE;
        updateScreen = true;
      }
      break;

    case REFLOW_STATE_STEP:
      Setpoint1 = profile.temperatureStep[currentStep - 1];
      if ((Input1 >= profile.temperatureStep[currentStep - 1]))
      {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        //reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_COMPLETE;
      }
      break;

    //считаем время перехода на следующий шаг
    case REFLOW_STATE_STEP_DWELL:
      if ((profile.profileSteps == currentStep) && (counter == 0))  //если достигли максимальной температуры профиля на  последнем шаге
      {
        // мелодия окончания пайки
        for (uint8_t j = 1; j < 5; j++) {
          beep_(100, 1046);
          delay(100);
        }
      }
      if (currentMillis - previousMillis > 1000)
      {
        previousMillis = currentMillis;
        counter = counter + 1;
      }
      if (counter == profile.dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
      {
        counter = 0;
        setpointRamp = 0;
        if (profile.profileSteps == currentStep)              //если достигли последнего шага
        {
          currentStep = 1;
          flag = false;
          TopStart = false;                                   //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
          D5_High;                                            //extefan включить
        } else {                                              //если шаг не последний
          currentStep++;                                      //переходим на следующий шаг
          flag = false;
          reflowState = REFLOW_STATE_STEP_RAMP;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 150)
      {
        ms_button =  millis();
        updateScreen = true;
        counter = 0;
        //reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_COMPLETE;
      }
      break;

    //завершение пайки
    case REFLOW_STATE_COMPLETE:
      Output1 = 0;
      Output2 = 0;
      D4_Low;                                                //intefan выключить
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;
      updateScreen = true;
      profile.Setpoint2 = Setpoint22;
      run_external_fan = true;
      previousMillis = millis();
      break;
  }

  //включение нагревателей
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    if (millis() > nextRead2)
    {
      nextRead2 = millis() + GRAPHICS_SAMPLING_TIME;
      if (curCount == 0) {
        lcd.setCursor(0, 0);
        lcd.print("*");
        lcd.setCursor(5, 0);
        lcd.write(byte(1));
        curCount = 1;
      } else {
        lcd.setCursor(0, 0);
        lcd.print(" ");
        lcd.setCursor(5, 0);
        lcd.print(" ");
        curCount = 0;
      }
    }

    if (millis() > nextRead3)
    {
      nextRead3 = millis() + SAMPLE_TIME;
      if (!TopStart) Output1 = 0;
      else Output1 = Pid1(Input1, Setpoint1, profile.kp1, profile.ki1, profile.kd1);
      if (!flag) Output2 = 0;
      else Output2 = Pid2(Input2, profile.Setpoint2, profile.kp2, profile.ki2, profile.kd2);
      lcdTemp();
    }
  } else {
    D11_Low;
    D12_Low;
  }
  End
}

void Dimming()
{
  OutPWR_TOP();
  OutPWR_BOTTOM();
  if (Secs >= 99)
  {
    Serial.println(buf);                                 //ВЫВОД графиков на ПК
    Secs = 0;
  } else Secs++;
}

void OutPWR_TOP() {
  reg1 = Output1 + er1;                                  //задание выходной мощности в %,в текущем шаге профиля, er- ошибка округления
  if (reg1 < 50) {
    D12_Low;                                              //пин через который осуществляется дискретное управление ВИ
    er1 = reg1;                                          //reg- переменная для расчетов
  } else {
    D12_High;                                             //пин через который осуществляется дискретное управление ВИ
    er1 = reg1 - 100;
  }
}

void OutPWR_BOTTOM() {
  reg2 = Output2 + er2;                                  //задание выходной мощности в %, er- ошибка округления
  if (reg2 < 50) {
    D11_Low;                                              //пин через который осуществляется дискретное управление НИ
    er2 = reg2;                                          //reg- переменная для расчетов
  } else {
    D11_High;                                             //пин через который осуществляется дискретное управление НИ
    er2 = reg2 - 100;
  }
}

void lcdTemp()
{
  // считываем данные с термопар
  Input1 = Input1 * 0.8 + 0.2 * max6675_read_temp(1);
  Input2 = Input2 * 0.6 + 0.2 * max6675_read_temp(0);
  tc1 = Input1;
  tc2 = Input2;
  sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile); // вывод на комп через UART
  lcd.setCursor(12, 0);
  if ((Input1 > 250)) {
    lcd.print("Er");
    lcd.print(" ");
  } else {
    if (tc1 < 100) lcd.print(Input1); else lcd.print(tc1);
    lcd.write(byte(0));
    lcd.print(" ");
  }
  lcd.setCursor(12, 1);
  if ((Input2) > 250) {
    lcd.print("Er");
    lcd.print(" ");
  } else {
    if (tc2 < 100) lcd.print(Input2); else lcd.print(tc2);
    lcd.write(byte(0));
    lcd.print(" ");
  }
}

//---MAX6675 functions---//
double max6675_read_temp (byte topORbottom)
{ byte i;
  int tmp = 0;
  if (topORbottom == 1)D9_Low; else D8_Low;                      //Останавливаем преобразование CS=0
  asm volatile(" nop"        "\n\t");                            //Задержка в 1 пустую команду
  for (i = 15; i > 0; i--)                                       //Счётчик для чтения, будем читать 16 бит
  { D7_High;                                                     //Устанавливаем вывод clk в единицу CLK=1
    asm volatile(" nop"        "\n\t");                          //Задержка в 1 пустую команду
    if ( D6_Read )                                               //Читаем бит c DO
      tmp |= (1 << i);                                           //если он не равен нулю, то записываем в соответствующий бит переменной tmp единицу
    D7_Low;                                                      //Устанавливаем вывод clk в ноль CLK=0
    asm volatile(" nop"        "\n\t");
  }                                                              //Задержка в 1 пустую команду
  D9_High;                                                       //Запускаем преобразование CS=1
  D8_High;                                                       //Запускаем преобразование CS=1
  if (tmp & 0x4) return 255;                                     //Проверяем 3 бит результата чтения и если он равен единице, значит произошла ошибка
  else return ((tmp >> 3)) * 0.25;                               //Результат сдвигаем на 3 позиции вправо и домножаем на 0,25
}
//---MAX6675 functions---//

byte Pid1(double temp, double ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e1 = (ust - temp);                                             //ошибка регулирования
  p1 =  (kP * e1) / 10.0;                                        //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e1) / 100.0; //И составляющая
  d1 = kd / 100.0 * (temp - ed);                                 //Д составляющая
  ed = temp;
  out = (p1 + integra - d1 < 0) ? 0 : (p1 + integra - d1 > profile.pwr_TOP) ? profile.pwr_TOP : p1 + integra - d1;
  if (Output1 >= 100 && e1 > 0 && integra > 0) integra -= e1;
  return out;
}

byte Pid2(double temp, double ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e2 = (ust - temp);                                             //ошибка регулирования
  p2 =  (kP * e2) ;                                              //П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e2) / 1000.0; //И составляющая
  d2 = (kd / 10)  * (temp - ed);                                 //Д составляющая
  ed = temp;
  out = (p2 + integra2 - d2 < 0) ? 0 : (p2 + integra2 - d2 > profile.pwr_BOTTOM) ? profile.pwr_BOTTOM : p2 + integra2 - d2;
  if (Output2 >= 100 && e2 > 0 && integra2 > 0)
    integra2 -= e2;
  return out;
}

void beep_(uint16_t dur, uint16_t frq)
{
  //dur=(1000/frq)*dur;                                          //расчет длительности бипа
  uint16_t per = 500000 / frq;                                   //длительность. полупериода в мкс
  dur = dur / (per / 250) * 2;
  for (uint16_t i = 0; i < dur; i++)
  {
    D10_High;
    delay_us(per);
    D10_Low;
    delay_us(per);
  }
}
